@@ -3,6 +3,8 @@
 require_once('test_base.php');
 require_once('test_util.php');
 
+use Google\Protobuf\Internal\CodedOutputStream;
+use Google\Protobuf\PrintOptions;
 use Google\Protobuf\RepeatedField;
 use Google\Protobuf\GPBType;
 use Foo\EmptyAnySerialization;
@@ -205,6 +207,68 @@ class EncodeDecodeTest extends TestBase
         $this->assertSame("a", $m->getRepeatedField()[0]->getValue());
     }
 
+    public function testDecodeEnumMapWithUnknownStringValueThrows()
+    {
+        $this->expectException(Exception::class);
+
+        $m = new TestMessage();
+        $m->mergeFromJsonString(
+          "{\"map_int32_enum\":{
+            \"1\": \"ONE\",
+            \"2\": \"UNKNOWN_ENUM\",
+            \"3\": \"ZERO\"
+          }}"
+        );
+    }
+
+    public function testDecodeEnumMapWithUnknownStringValueIgnored()
+    {
+        $m = new TestMessage();
+        $m->mergeFromJsonString(
+          "{\"map_int32_enum\":{
+            \"1\": \"ONE\",
+            \"2\": \"UNKNOWN_ENUM\",
+            \"3\": \"ZERO\"
+          }}",
+          true
+        );
+
+        $this->assertSame(TestEnum::ONE, $m->getMapInt32Enum()["1"]);
+        $this->assertSame(TestEnum::ZERO, $m->getMapInt32Enum()["3"]);
+        $this->assertFalse($m->getMapInt32Enum()->offsetExists(2));
+    }
+
+    public function testDecodeRepeatedEnumWithUnknownStringValueThrows()
+    {
+        $this->expectException(Exception::class);
+
+        $m = new TestMessage();
+        $m->mergeFromJsonString(
+          "{\"repeated_enum\":[
+            \"ONE\",
+            \"UNKNOWN_ENUM\",
+            \"ZERO\"
+          ]}"
+        );
+    }
+
+    public function testDecodeRepeatedEnumWithUnknownStringValueIgnored()
+    {
+        $m = new TestMessage();
+        $m->mergeFromJsonString(
+          "{\"repeated_enum\":[
+            \"ONE\",
+            \"UNKNOWN_ENUM\",
+            \"ZERO\"
+          ]}",
+          true
+        );
+
+        $this->assertSame(2, count($m->getRepeatedEnum()));
+        $this->assertSame(TestEnum::ONE, $m->getRepeatedEnum()[0]);
+        $this->assertSame(TestEnum::ZERO, $m->getRepeatedEnum()[1]);
+    }
+
     public function testDecodeMapStringValue()
     {
         $m = new TestStringValue();
@@ -683,6 +747,21 @@ class EncodeDecodeTest extends TestBase
         $m->mergeFromString(hex2bin('7201'));
     }
 
+    public function testDecodeInvalidStringDataBadUtf8()
+    {
+        $this->expectException(Exception::class);
+
+        $m = new TestMessage();
+        $m->mergeFromString(hex2bin('720180'));
+    }
+
+    public function testDecodeValidStringData()
+    {
+        $m = new TestMessage();
+        $m->mergeFromString(hex2bin('720161'));
+        $this->assertSame('a', $m->getOptionalString());
+    }
+
     public function testDecodeInvalidBytesLengthMiss()
     {
         $this->expectException(Exception::class);
@@ -1542,4 +1621,90 @@ class EncodeDecodeTest extends TestBase
         $this->assertInstanceOf(EmptyAnySerialization::class, $m);
         $this->assertEquals('', $m->getA());
     }
+
+    public function testSerializeToJsonStringNoOption()
+    {
+        $m = new TestMessage();
+
+        $m->setOneofEnum(TestEnum::ONE);
+        $data = $m->serializeToJsonString();
+        $this->assertSame('{"oneofEnum":"ONE"}', $data);
+
+        $n = new TestMessage();
+        $n->mergeFromJsonString($data);
+        $this->assertSame("oneof_enum", $n->getMyOneof());
+        $this->assertSame(TestEnum::ONE, $n->getOneofEnum());
+    }
+
+    /**
+     * @dataProvider serializeToJsonStringOptionsProvider
+     */
+    public function testSerializeToJsonStringWithOptions(int|bool|null $flags, string $expected)
+    {
+        $m = new TestMessage();
+
+        $m->setOneofEnum(TestEnum::ONE);
+        $data = $m->serializeToJsonString($flags);
+        $this->assertSame($expected, $data);
+
+        $n = new TestMessage();
+        $n->mergeFromJsonString($data);
+        $this->assertSame("oneof_enum", $n->getMyOneof());
+        $this->assertSame(TestEnum::ONE, $n->getOneofEnum());
+    }
+
+    public static function serializeToJsonStringOptionsProvider(): array
+    {
+        return [
+            'null' => [
+                null,
+                '{"oneofEnum":"ONE"}',
+            ],
+            'boolean false (BC)' => [
+                false,
+                '{"oneofEnum":"ONE"}',
+            ],
+            'boolean true (BC)' => [
+                true,
+                '{"oneof_enum":"ONE"}',
+            ],
+            'default int' => [
+                0,
+                '{"oneofEnum":"ONE"}',
+            ],
+            'as int' => [
+                PrintOptions::ALWAYS_PRINT_ENUMS_AS_INTS,
+                '{"oneofEnum":1}',
+            ],
+            'preserve' => [
+                PrintOptions::PRESERVE_PROTO_FIELD_NAMES,
+                '{"oneof_enum":"ONE"}',
+            ],
+            'as int + preserve' => [
+                PrintOptions::ALWAYS_PRINT_ENUMS_AS_INTS | PrintOptions::PRESERVE_PROTO_FIELD_NAMES,
+                '{"oneof_enum":1}',
+            ],
+        ];
+    }
+
+    /**
+     * @dataProvider preserveProtoFieldNamesProvider
+     * Backwards Compatibility test for boolean parameter to serializeToJsonString
+     */
+    public function testJsonStringBCPreserveProtoFieldNames(bool $value, $expected)
+    {
+        $m = new TestMessage();
+
+        $m->setOneofEnum(TestEnum::ONE);
+        $data = $m->serializeToJsonString($value);
+        $this->assertSame($expected, $data);
+    }
+
+    public static function preserveProtoFieldNamesProvider(): array
+    {
+        return [
+            [true, '{"oneof_enum":"ONE"}'],
+            [false, '{"oneofEnum":"ONE"}'],
+        ];
+    }
 }
