@@ -18,17 +18,18 @@ from google.protobuf import descriptor_pool
 from google.protobuf import symbol_database
 from google.protobuf import text_format
 from google.protobuf.internal import api_implementation
+from google.protobuf.internal import test_proto2_pb2
 from google.protobuf.internal import test_util
 from google.protobuf.internal import testing_refleaks
 
-from google.protobuf.internal import _parameterized
-from google.protobuf import unittest_legacy_features_pb2
+from absl.testing import parameterized
 from google.protobuf import unittest_custom_options_pb2
 from google.protobuf import unittest_features_pb2
 from google.protobuf import unittest_import_pb2
+from google.protobuf import unittest_legacy_features_pb2
 from google.protobuf import unittest_pb2
-from google.protobuf import unittest_proto3_pb2
 from google.protobuf import unittest_proto3_extensions_pb2
+from google.protobuf import unittest_proto3_pb2
 
 
 TEST_EMPTY_MESSAGE_DESCRIPTOR_ASCII = """
@@ -37,7 +38,7 @@ name: 'TestEmptyMessage'
 
 TEST_FILE_DESCRIPTOR_DEBUG = """syntax = "proto2";
 
-package protobuf_unittest;
+package proto2_unittest;
 
 message NestedMessage {
   enum ForeignEnum {
@@ -53,7 +54,7 @@ message ResponseMessage {
 }
 
 service DescriptorTestService {
-  rpc CallMethod(.protobuf_unittest.NestedMessage) returns (.protobuf_unittest.ResponseMessage);
+  rpc CallMethod(.proto2_unittest.NestedMessage) returns (.proto2_unittest.ResponseMessage);
 }
 
 """
@@ -68,7 +69,7 @@ class DescriptorTest(unittest.TestCase):
   def setUp(self):
     file_proto = descriptor_pb2.FileDescriptorProto(
         name='some/filename/some.proto',
-        package='protobuf_unittest')
+        package='proto2_unittest')
     message_proto = file_proto.message_type.add(
         name='NestedMessage')
     message_proto.field.add(
@@ -87,8 +88,8 @@ class DescriptorTest(unittest.TestCase):
     service_proto = file_proto.service.add(name='DescriptorTestService')
     method_proto = service_proto.method.add(
         name='CallMethod',
-        input_type='.protobuf_unittest.NestedMessage',
-        output_type='.protobuf_unittest.ResponseMessage')
+        input_type='.proto2_unittest.NestedMessage',
+        output_type='.proto2_unittest.ResponseMessage')
 
     # Note: Calling DescriptorPool.Add() multiple times with the same file only
     # works if the input is canonical; in particular, all type names must be
@@ -235,13 +236,6 @@ class DescriptorTest(unittest.TestCase):
   def testContainingServiceFixups(self):
     self.assertEqual(self.my_service, self.my_method.containing_service)
 
-  @unittest.skipIf(
-      api_implementation.Type() == 'python',
-      'GetDebugString is only available with the cpp implementation',
-  )
-  def testGetDebugString(self):
-    self.assertEqual(self.my_file.GetDebugString(), TEST_FILE_DESCRIPTOR_DEBUG)
-
   def testGetOptions(self):
     self.assertEqual(self.my_enum.GetOptions(),
                      descriptor_pb2.EnumOptions())
@@ -295,6 +289,7 @@ class DescriptorTest(unittest.TestCase):
     service_options = service_descriptor.GetOptions()
     service_opt1 = unittest_custom_options_pb2.service_opt1
     self.assertEqual(-9876543210, service_options.Extensions[service_opt1])
+    self.assertTrue(method_descriptor.has_options)
     method_options = method_descriptor.GetOptions()
     method_opt1 = unittest_custom_options_pb2.method_opt1
     self.assertEqual(unittest_custom_options_pb2.METHODOPT1_VAL2,
@@ -537,7 +532,7 @@ class DescriptorTest(unittest.TestCase):
 
   def testFileDescriptor(self):
     self.assertEqual(self.my_file.name, 'some/filename/some.proto')
-    self.assertEqual(self.my_file.package, 'protobuf_unittest')
+    self.assertEqual(self.my_file.package, 'proto2_unittest')
     self.assertEqual(self.my_file.pool, self.pool)
     self.assertFalse(self.my_file.has_options)
     file_proto = descriptor_pb2.FileDescriptorProto()
@@ -549,7 +544,8 @@ class DescriptorTest(unittest.TestCase):
 
   @unittest.skipIf(
       api_implementation.Type() == 'python',
-      'Immutability of descriptors is only enforced in v2 implementation')
+      'Immutability of descriptors is only enforced in c++ and upb backends',
+  )
   def testImmutableCppDescriptor(self):
     file_descriptor = unittest_pb2.DESCRIPTOR
     message_descriptor = unittest_pb2.TestAllTypes.DESCRIPTOR
@@ -574,19 +570,34 @@ class DescriptorTest(unittest.TestCase):
       enum_descriptor.has_options = False
     with self.assertRaises(AttributeError) as e:
       message_descriptor.has_options = True
-    self.assertEqual('attribute is not writable: has_options',
-                     str(e.exception))
+
+    if api_implementation.Type() == 'cpp':
+      self.assertEqual(
+          'attribute is not writable: has_options', str(e.exception)
+      )
+    else:
+      self.assertEqual(api_implementation.Type(), 'upb')
+      self.assertEqual(
+          "attribute 'has_options' of "
+          "'google._upb._message.Descriptor' "
+          'objects is not writable',
+          str(e.exception),
+      )
 
   def testDefault(self):
     message_descriptor = unittest_pb2.TestAllTypes.DESCRIPTOR
     field = message_descriptor.fields_by_name['repeated_int32']
     self.assertEqual(field.default_value, [])
+    self.assertTrue(field.is_repeated)
     field = message_descriptor.fields_by_name['repeated_nested_message']
     self.assertEqual(field.default_value, [])
+    self.assertTrue(field.is_repeated)
     field = message_descriptor.fields_by_name['optionalgroup']
     self.assertEqual(field.default_value, None)
+    self.assertFalse(field.is_required)
     field = message_descriptor.fields_by_name['optional_nested_message']
     self.assertEqual(field.default_value, None)
+    self.assertFalse(field.is_required)
 
 
 class NewDescriptorTest(DescriptorTest):
@@ -603,7 +614,7 @@ class GeneratedDescriptorTest(unittest.TestCase):
     # Basic properties
     self.assertEqual(message_descriptor.name, 'TestAllTypes')
     self.assertEqual(message_descriptor.full_name,
-                     'protobuf_unittest.TestAllTypes')
+                     'proto2_unittest.TestAllTypes')
     # Test equality and hashability
     self.assertEqual(message_descriptor, message_descriptor)
     self.assertEqual(message_descriptor.fields[0].containing_type,
@@ -627,7 +638,7 @@ class GeneratedDescriptorTest(unittest.TestCase):
     self.assertEqual(field_descriptor.name, 'optional_int32')
     self.assertEqual(field_descriptor.camelcase_name, 'optionalInt32')
     self.assertEqual(field_descriptor.full_name,
-                     'protobuf_unittest.TestAllTypes.optional_int32')
+                     'proto2_unittest.TestAllTypes.optional_int32')
     self.assertEqual(field_descriptor.containing_type.name, 'TestAllTypes')
     self.assertEqual(field_descriptor.file, unittest_pb2.DESCRIPTOR)
     # Test equality and hashability
@@ -643,6 +654,7 @@ class GeneratedDescriptorTest(unittest.TestCase):
     self.assertIn(field_descriptor, {field_descriptor: None})
     self.assertEqual(None, field_descriptor.extension_scope)
     self.assertEqual(None, field_descriptor.enum_type)
+    self.assertFalse(field_descriptor.is_required)
     self.assertTrue(field_descriptor.has_presence)
     if api_implementation.Type() == 'cpp':
       # For test coverage only
@@ -786,6 +798,7 @@ class GeneratedDescriptorTest(unittest.TestCase):
     self.assertIs(service_descriptor.file, unittest_pb2.DESCRIPTOR)
     self.assertEqual(service_descriptor.index, 0)
     self.CheckDescriptorMapping(service_descriptor.methods_by_name)
+    self.assertFalse(service_descriptor.has_options)
 
   def testOneofDescriptor(self):
     message_descriptor = unittest_pb2.TestAllTypes.DESCRIPTOR
@@ -793,7 +806,7 @@ class GeneratedDescriptorTest(unittest.TestCase):
     self.assertFalse(oneof_descriptor.has_options)
     self.assertEqual(message_descriptor, oneof_descriptor.containing_type)
     self.assertEqual('oneof_field', oneof_descriptor.name)
-    self.assertEqual('protobuf_unittest.TestAllTypes.oneof_field',
+    self.assertEqual('proto2_unittest.TestAllTypes.oneof_field',
                      oneof_descriptor.full_name)
     self.assertEqual(0, oneof_descriptor.index)
 
@@ -856,7 +869,7 @@ class DescriptorCopyToProtoTest(unittest.TestCase):
         number: 1
         label: 1  # Optional
         type: 11  # TYPE_MESSAGE
-        type_name: '.protobuf_unittest.TestAllTypes.NestedMessage'
+        type_name: '.proto2_unittest.TestAllTypes.NestedMessage'
       >
       """
 
@@ -921,7 +934,7 @@ class DescriptorCopyToProtoTest(unittest.TestCase):
         number: 3
         label: LABEL_OPTIONAL
         type: TYPE_MESSAGE
-        type_name: ".protobuf_unittest.TestAllTypes.NestedMessage"
+        type_name: ".proto2_unittest.TestAllTypes.NestedMessage"
         options {
           deprecated: true
         }
@@ -941,7 +954,7 @@ class DescriptorCopyToProtoTest(unittest.TestCase):
         number: 5
         label: LABEL_OPTIONAL
         type: TYPE_MESSAGE
-        type_name: ".protobuf_unittest.TestDeprecatedFields"
+        type_name: ".proto2_unittest.TestDeprecatedFields"
       }
       oneof_decl {
         name: "oneof_fields"
@@ -992,7 +1005,7 @@ class DescriptorCopyToProtoTest(unittest.TestCase):
   def testCopyToProto_FileDescriptor(self):
     UNITTEST_IMPORT_FILE_DESCRIPTOR_ASCII = ("""
       name: 'google/protobuf/unittest_import.proto'
-      package: 'protobuf_unittest_import'
+      package: 'proto2_unittest_import'
       dependency: 'google/protobuf/unittest_import_public.proto'
       message_type: <
         name: 'ImportMessage'
@@ -1053,13 +1066,13 @@ class DescriptorCopyToProtoTest(unittest.TestCase):
       name: 'TestService'
       method: <
         name: 'Foo'
-        input_type: '.protobuf_unittest.FooRequest'
-        output_type: '.protobuf_unittest.FooResponse'
+        input_type: '.proto2_unittest.FooRequest'
+        output_type: '.proto2_unittest.FooResponse'
       >
       method: <
         name: 'Bar'
-        input_type: '.protobuf_unittest.BarRequest'
-        output_type: '.protobuf_unittest.BarResponse'
+        input_type: '.proto2_unittest.BarRequest'
+        output_type: '.proto2_unittest.BarResponse'
       >
       """
     self._InternalTestCopyToProto(
@@ -1070,8 +1083,8 @@ class DescriptorCopyToProtoTest(unittest.TestCase):
   def testCopyToProto_MethodDescriptor(self):
     expected_ascii = """
       name: 'Foo'
-      input_type: '.protobuf_unittest.FooRequest'
-      output_type: '.protobuf_unittest.FooResponse'
+      input_type: '.proto2_unittest.FooRequest'
+      output_type: '.proto2_unittest.FooResponse'
     """
     method_descriptor = unittest_pb2.TestService.DESCRIPTOR.FindMethodByName(
         'Foo')
@@ -1225,9 +1238,9 @@ class MakeDescriptorTest(unittest.TestCase):
 
 
 @testing_refleaks.TestCase
-class FeaturesTest(_parameterized.TestCase):
+class FeaturesTest(parameterized.TestCase):
 
-  @_parameterized.named_parameters([
+  @parameterized.named_parameters([
       ('File', lambda: descriptor_pb2.DESCRIPTOR),
       ('Message', lambda: descriptor_pb2.FeatureSet.DESCRIPTOR),
       (
@@ -1280,10 +1293,7 @@ class FeaturesTest(_parameterized.TestCase):
 
   def testLegacyRequiredTransform(self):
     desc = unittest_legacy_features_pb2.TestEditionsMessage.DESCRIPTOR
-    self.assertEqual(
-        desc.fields_by_name['required_field'].label,
-        descriptor.FieldDescriptor.LABEL_REQUIRED,
-    )
+    self.assertTrue(desc.fields_by_name['required_field'].is_required)
 
   def testLegacyGroupTransform(self):
     desc = unittest_legacy_features_pb2.TestEditionsMessage.DESCRIPTOR
@@ -1325,7 +1335,7 @@ class FeaturesTest(_parameterized.TestCase):
     )
 
   def testProto2Defaults(self):
-    features = unittest_pb2.TestAllTypes.DESCRIPTOR.fields_by_name[
+    features = test_proto2_pb2.TestProto2.DESCRIPTOR.fields_by_name[
         'optional_int32'
     ]._GetFeatures()
     fs = descriptor_pb2.FeatureSet
@@ -1393,7 +1403,7 @@ class FeatureInheritanceTest(unittest.TestCase):
     super(FeatureInheritanceTest, self).setUp()
     self.file_proto = descriptor_pb2.FileDescriptorProto(
         name='some/filename/some.proto',
-        package='protobuf_unittest',
+        package='proto2_unittest',
         edition=descriptor_pb2.Edition.EDITION_2023,
         syntax='editions',
     )
@@ -1402,7 +1412,7 @@ class FeatureInheritanceTest(unittest.TestCase):
         number=10,
         type=descriptor_pb2.FieldDescriptorProto.TYPE_INT32,
         label=descriptor_pb2.FieldDescriptorProto.LABEL_OPTIONAL,
-        extendee='.protobuf_unittest.TopMessage',
+        extendee='.proto2_unittest.TopMessage',
     )
     self.top_enum_proto = self.file_proto.enum_type.add(name='TopEnum')
     self.enum_value_proto = self.top_enum_proto.value.add(
@@ -1421,7 +1431,7 @@ class FeatureInheritanceTest(unittest.TestCase):
         number=11,
         type=descriptor_pb2.FieldDescriptorProto.TYPE_INT32,
         label=descriptor_pb2.FieldDescriptorProto.LABEL_OPTIONAL,
-        extendee='.protobuf_unittest.TopMessage',
+        extendee='.proto2_unittest.TopMessage',
     )
     self.nested_message_proto = self.top_message_proto.nested_type.add(
         name='NestedMessage'
@@ -1442,8 +1452,8 @@ class FeatureInheritanceTest(unittest.TestCase):
     self.service_proto = self.file_proto.service.add(name='TestService')
     self.method_proto = self.service_proto.method.add(
         name='CallMethod',
-        input_type='.protobuf_unittest.TopMessage',
-        output_type='.protobuf_unittest.TopMessage',
+        input_type='.proto2_unittest.TopMessage',
+        output_type='.proto2_unittest.TopMessage',
     )
 
   def BuildPool(self):
@@ -1474,16 +1484,17 @@ class FeatureInheritanceTest(unittest.TestCase):
     file = descriptor_pb2.FileDescriptorProto()
     descriptor_pb2.DESCRIPTOR.CopyToProto(file)
     ret.pool.Add(file)
+    file.Clear()
     unittest_features_pb2.DESCRIPTOR.CopyToProto(file)
     ret.pool.Add(file)
 
     ret.file = ret.pool.AddSerializedFile(self.file_proto.SerializeToString())
     ret.top_message = ret.pool.FindMessageTypeByName(
-        'protobuf_unittest.TopMessage'
+        'proto2_unittest.TopMessage'
     )
-    ret.top_enum = ret.pool.FindEnumTypeByName('protobuf_unittest.TopEnum')
+    ret.top_enum = ret.pool.FindEnumTypeByName('proto2_unittest.TopEnum')
     ret.top_extension = ret.pool.FindExtensionByName(
-        'protobuf_unittest.top_extension'
+        'proto2_unittest.top_extension'
     )
     ret.nested_message = ret.top_message.nested_types_by_name['NestedMessage']
     ret.nested_enum = ret.top_message.enum_types_by_name['NestedEnum']
@@ -1494,7 +1505,7 @@ class FeatureInheritanceTest(unittest.TestCase):
     ret.oneof = ret.top_message.oneofs_by_name['Oneof']
     ret.oneof_field = ret.top_message.fields_by_name['oneof_field']
     ret.enum_value = ret.top_enum.values_by_name['TOP_VALUE']
-    ret.service = ret.pool.FindServiceByName('protobuf_unittest.TestService')
+    ret.service = ret.pool.FindServiceByName('proto2_unittest.TestService')
     ret.method = ret.service.methods_by_name['CallMethod']
     return ret
 
