@@ -29,6 +29,7 @@ from google.protobuf import unittest_pb2
 try:
   # New module in Python 3.9:
   import zoneinfo  # pylint:disable=g-import-not-at-top
+
   _TZ_JAPAN = zoneinfo.ZoneInfo('Japan')
   _TZ_PACIFIC = zoneinfo.ZoneInfo('US/Pacific')
   has_zoneinfo = True
@@ -124,43 +125,35 @@ class TimeUtilTest(TimeUtilTestBase):
   def testTimestampIntegerConversion(self):
     message = timestamp_pb2.Timestamp()
     message.FromNanoseconds(1)
-    self.assertEqual('1970-01-01T00:00:00.000000001Z',
-                     message.ToJsonString())
+    self.assertEqual('1970-01-01T00:00:00.000000001Z', message.ToJsonString())
     self.assertEqual(1, message.ToNanoseconds())
 
     message.FromNanoseconds(-1)
-    self.assertEqual('1969-12-31T23:59:59.999999999Z',
-                     message.ToJsonString())
+    self.assertEqual('1969-12-31T23:59:59.999999999Z', message.ToJsonString())
     self.assertEqual(-1, message.ToNanoseconds())
 
     message.FromMicroseconds(1)
-    self.assertEqual('1970-01-01T00:00:00.000001Z',
-                     message.ToJsonString())
+    self.assertEqual('1970-01-01T00:00:00.000001Z', message.ToJsonString())
     self.assertEqual(1, message.ToMicroseconds())
 
     message.FromMicroseconds(-1)
-    self.assertEqual('1969-12-31T23:59:59.999999Z',
-                     message.ToJsonString())
+    self.assertEqual('1969-12-31T23:59:59.999999Z', message.ToJsonString())
     self.assertEqual(-1, message.ToMicroseconds())
 
     message.FromMilliseconds(1)
-    self.assertEqual('1970-01-01T00:00:00.001Z',
-                     message.ToJsonString())
+    self.assertEqual('1970-01-01T00:00:00.001Z', message.ToJsonString())
     self.assertEqual(1, message.ToMilliseconds())
 
     message.FromMilliseconds(-1)
-    self.assertEqual('1969-12-31T23:59:59.999Z',
-                     message.ToJsonString())
+    self.assertEqual('1969-12-31T23:59:59.999Z', message.ToJsonString())
     self.assertEqual(-1, message.ToMilliseconds())
 
     message.FromSeconds(1)
-    self.assertEqual('1970-01-01T00:00:01Z',
-                     message.ToJsonString())
+    self.assertEqual('1970-01-01T00:00:01Z', message.ToJsonString())
     self.assertEqual(1, message.ToSeconds())
 
     message.FromSeconds(-1)
-    self.assertEqual('1969-12-31T23:59:59Z',
-                     message.ToJsonString())
+    self.assertEqual('1969-12-31T23:59:59Z', message.ToJsonString())
     self.assertEqual(-1, message.ToSeconds())
 
     message.FromNanoseconds(1999)
@@ -175,33 +168,27 @@ class TimeUtilTest(TimeUtilTestBase):
   def testDurationIntegerConversion(self):
     message = duration_pb2.Duration()
     message.FromNanoseconds(1)
-    self.assertEqual('0.000000001s',
-                     message.ToJsonString())
+    self.assertEqual('0.000000001s', message.ToJsonString())
     self.assertEqual(1, message.ToNanoseconds())
 
     message.FromNanoseconds(-1)
-    self.assertEqual('-0.000000001s',
-                     message.ToJsonString())
+    self.assertEqual('-0.000000001s', message.ToJsonString())
     self.assertEqual(-1, message.ToNanoseconds())
 
     message.FromMicroseconds(1)
-    self.assertEqual('0.000001s',
-                     message.ToJsonString())
+    self.assertEqual('0.000001s', message.ToJsonString())
     self.assertEqual(1, message.ToMicroseconds())
 
     message.FromMicroseconds(-1)
-    self.assertEqual('-0.000001s',
-                     message.ToJsonString())
+    self.assertEqual('-0.000001s', message.ToJsonString())
     self.assertEqual(-1, message.ToMicroseconds())
 
     message.FromMilliseconds(1)
-    self.assertEqual('0.001s',
-                     message.ToJsonString())
+    self.assertEqual('0.001s', message.ToJsonString())
     self.assertEqual(1, message.ToMilliseconds())
 
     message.FromMilliseconds(-1)
-    self.assertEqual('-0.001s',
-                     message.ToJsonString())
+    self.assertEqual('-0.001s', message.ToJsonString())
     self.assertEqual(-1, message.ToMilliseconds())
 
     message.FromSeconds(1)
@@ -209,8 +196,7 @@ class TimeUtilTest(TimeUtilTestBase):
     self.assertEqual(1, message.ToSeconds())
 
     message.FromSeconds(-1)
-    self.assertEqual('-1s',
-                     message.ToJsonString())
+    self.assertEqual('-1s', message.ToJsonString())
     self.assertEqual(-1, message.ToSeconds())
 
     # Test truncation behavior.
@@ -239,8 +225,9 @@ class TimeUtilTest(TimeUtilTestBase):
     self.assertEqual(1, message.seconds)
     self.assertEqual(999_000_000, message.nanos)
 
-    self.assertEqual(datetime.datetime(1970, 1, 1, 0, 0, 1, 999000),
-                     message.ToDatetime())
+    self.assertEqual(
+        datetime.datetime(1970, 1, 1, 0, 0, 1, 999000), message.ToDatetime()
+    )
 
   def testTimezoneNaiveDatetimeConversionWhereTimestampLosesPrecision(self):
     ts = timestamp_pb2.Timestamp()
@@ -298,7 +285,8 @@ class TimeUtilTest(TimeUtilTestBase):
     self.assertEqual(original_datetime, aware_datetime)
     self.assertEqual(
         datetime.datetime(1970, 1, 1, 2, tzinfo=datetime.timezone.utc),
-        aware_datetime)
+        aware_datetime,
+    )
     self.assertEqual(tzinfo, aware_datetime.tzinfo)
 
   @unittest.skipIf(
@@ -476,13 +464,11 @@ class TimeUtilTest(TimeUtilTestBase):
   def testNanosOneSecond(self):
     tz = _TZ_PACIFIC
     ts = timestamp_pb2.Timestamp(nanos=1_000_000_000)
-    self.assertRaisesRegex(ValueError, 'Timestamp is not valid',
-                           ts.ToDatetime)
+    self.assertRaisesRegex(ValueError, 'Timestamp is not valid', ts.ToDatetime)
 
   def testNanosNegativeOneSecond(self):
     ts = timestamp_pb2.Timestamp(nanos=-1_000_000_000)
-    self.assertRaisesRegex(ValueError, 'Timestamp is not valid',
-                           ts.ToDatetime)
+    self.assertRaisesRegex(ValueError, 'Timestamp is not valid', ts.ToDatetime)
 
   def testTimedeltaConversion(self):
     message = duration_pb2.Duration()
@@ -509,27 +495,43 @@ class TimeUtilTest(TimeUtilTestBase):
   def testInvalidTimestamp(self):
     message = timestamp_pb2.Timestamp()
     self.assertRaisesRegex(
-        ValueError, 'Failed to parse timestamp: missing valid timezone offset.',
-        message.FromJsonString, '')
-    self.assertRaisesRegex(
-        ValueError, 'Failed to parse timestamp: invalid trailing data '
-        '1970-01-01T00:00:01Ztrail.', message.FromJsonString,
-        '1970-01-01T00:00:01Ztrail')
-    self.assertRaisesRegex(
-        ValueError, 'time data \'10000-01-01T00:00:00\' does not match'
-        ' format \'%Y-%m-%dT%H:%M:%S\'', message.FromJsonString,
-        '10000-01-01T00:00:00.00Z')
-    self.assertRaisesRegex(
-        ValueError, 'nanos 0123456789012 more than 9 fractional digits.',
-        message.FromJsonString, '1970-01-01T00:00:00.0123456789012Z')
+        ValueError,
+        'Failed to parse timestamp: missing valid timezone offset.',
+        message.FromJsonString,
+        '',
+    )
     self.assertRaisesRegex(
         ValueError,
-        (r'Invalid timezone offset value: \+08.'),
+        'Failed to parse timestamp: invalid trailing data '
+        '1970-01-01T00:00:01Ztrail.',
+        message.FromJsonString,
+        '1970-01-01T00:00:01Ztrail',
+    )
+    self.assertRaisesRegex(
+        ValueError,
+        "time data '10000-01-01T00:00:00' does not match"
+        " format '%Y-%m-%dT%H:%M:%S'",
+        message.FromJsonString,
+        '10000-01-01T00:00:00.00Z',
+    )
+    self.assertRaisesRegex(
+        ValueError,
+        'nanos 0123456789012 more than 9 fractional digits.',
+        message.FromJsonString,
+        '1970-01-01T00:00:00.0123456789012Z',
+    )
+    self.assertRaisesRegex(
+        ValueError,
+        r'Invalid timezone offset value: \+08.',
         message.FromJsonString,
         '1972-01-01T01:00:00.01+08',
     )
-    self.assertRaisesRegex(ValueError, 'year (0 )?is out of range',
-                           message.FromJsonString, '0000-01-01T00:00:00Z')
+    self.assertRaisesRegex(
+        ValueError,
+        'year (0 )?is out of range',
+        message.FromJsonString,
+        '0000-01-01T00:00:00Z',
+    )
     message.seconds = 253402300800
     self.assertRaisesRegex(ValueError, 'Timestamp is not valid',
                            message.ToJsonString)
@@ -550,36 +552,57 @@ class TimeUtilTest(TimeUtilTestBase):
 
   def testInvalidDuration(self):
     message = duration_pb2.Duration()
-    self.assertRaisesRegex(ValueError, 'Duration must end with letter "s": 1.',
-                           message.FromJsonString, '1')
-    self.assertRaisesRegex(ValueError, 'Couldn\'t parse duration: 1...2s.',
-                           message.FromJsonString, '1...2s')
+    self.assertRaisesRegex(
+        ValueError,
+        'Duration must end with letter "s": 1.',
+        message.FromJsonString,
+        '1',
+    )
+    self.assertRaisesRegex(
+        ValueError,
+        "Couldn't parse duration: 1...2s.",
+        message.FromJsonString,
+        '1...2s',
+    )
     text = '-315576000001.000000000s'
     self.assertRaisesRegex(
         ValueError,
         r'Duration is not valid\: Seconds -315576000001 must be in range'
-        r' \[-315576000000\, 315576000000\].', message.FromJsonString, text)
+        r' \[-315576000000\, 315576000000\].',
+        message.FromJsonString,
+        text,
+    )
     text = '315576000001.000000000s'
     self.assertRaisesRegex(
         ValueError,
         r'Duration is not valid\: Seconds 315576000001 must be in range'
-        r' \[-315576000000\, 315576000000\].', message.FromJsonString, text)
+        r' \[-315576000000\, 315576000000\].',
+        message.FromJsonString,
+        text,
+    )
     message.seconds = -315576000001
     message.nanos = 0
     self.assertRaisesRegex(
         ValueError,
         r'Duration is not valid\: Seconds -315576000001 must be in range'
-        r' \[-315576000000\, 315576000000\].', message.ToJsonString)
+        r' \[-315576000000\, 315576000000\].',
+        message.ToJsonString,
+    )
     message.seconds = 0
     message.nanos = 999999999 + 1
     self.assertRaisesRegex(
-        ValueError, r'Duration is not valid\: Nanos 1000000000 must be in range'
-        r' \[-999999999\, 999999999\].', message.ToJsonString)
+        ValueError,
+        r'Duration is not valid\: Nanos 1000000000 must be in range'
+        r' \[-999999999\, 999999999\].',
+        message.ToJsonString,
+    )
     message.seconds = -1
     message.nanos = 1
-    self.assertRaisesRegex(ValueError,
-                           r'Duration is not valid\: Sign mismatch.',
-                           message.ToJsonString)
+    self.assertRaisesRegex(
+        ValueError,
+        r'Duration is not valid\: Sign mismatch.',
+        message.ToJsonString,
+    )
     msg = well_known_types_test_pb2.WKTMessage()
     with self.assertRaises(AttributeError):
       msg.optional_duration = 1
@@ -683,6 +706,11 @@ class TimeUtilTest(TimeUtilTestBase):
 
 class StructTest(unittest.TestCase):
 
+  def testEmptyDict(self):
+    # in operator for empty initialized struct
+    msg = well_known_types_test_pb2.WKTMessage(optional_struct={})
+    self.assertNotIn('key', msg.optional_struct)
+
   def testStruct(self):
     struct = struct_pb2.Struct()
     self.assertIsInstance(struct, collections_abc.Mapping)
@@ -708,8 +736,10 @@ class StructTest(unittest.TestCase):
     self.assertEqual(11, struct['key4']['subkey'])
     inner_struct = struct_class()
     inner_struct['subkey2'] = 9
-    self.assertEqual([6, 'seven', True, False, None, inner_struct],
-                     list(struct['key5'].items()))
+    self.assertEqual(
+        [6, 'seven', True, False, None, inner_struct],
+        list(struct['key5'].items()),
+    )
     self.assertEqual({}, dict(struct['key6']['subkey'].fields))
     self.assertEqual([2, False], list(struct['key7'].items()))
 
@@ -738,8 +768,10 @@ class StructTest(unittest.TestCase):
     self.assertEqual('abc', struct2['key2'])
     self.assertIs(True, struct2['key3'])
     self.assertEqual(11, struct2['key4']['subkey'])
-    self.assertEqual([6, 'seven', True, False, None, inner_struct],
-                     list(struct2['key5'].items()))
+    self.assertEqual(
+        [6, 'seven', True, False, None, inner_struct],
+        list(struct2['key5'].items()),
+    )
 
     struct_list = struct2['key5']
     self.assertEqual(6, struct_list[0])
@@ -753,8 +785,9 @@ class StructTest(unittest.TestCase):
     self.assertEqual(7, struct_list[1])
 
     struct_list.add_list().extend([1, 'two', True, False, None])
-    self.assertEqual([1, 'two', True, False, None],
-                     list(struct_list[6].items()))
+    self.assertEqual(
+        [1, 'two', True, False, None], list(struct_list[6].items())
+    )
     struct_list.extend([{'nested_struct': 30}, ['nested_list', 99], {}, []])
     self.assertEqual(11, len(struct_list.values))
     self.assertEqual(30, struct_list[7]['nested_struct'])
@@ -799,8 +832,9 @@ class StructTest(unittest.TestCase):
     self.assertEqual(6, len(struct['key5']))
     del struct['key5'][1]
     self.assertEqual(5, len(struct['key5']))
-    self.assertEqual([6, True, False, None, inner_struct],
-                     list(struct['key5'].items()))
+    self.assertEqual(
+        [6, True, False, None, inner_struct], list(struct['key5'].items())
+    )
 
   def testInOperator(self):
     # in operator for Struct
@@ -921,7 +955,7 @@ class StructTest(unittest.TestCase):
         'key6': [['nested_list', True]],
         'empty_struct': {},
         'empty_list': [],
-        'tuple': ((3,2), ())
+        'tuple': ((3, 2), ()),
     }
     struct.update(dictionary)
     self.assertEqual(5, struct['key1'])
@@ -930,8 +964,10 @@ class StructTest(unittest.TestCase):
     self.assertEqual(11, struct['key4']['subkey'])
     inner_struct = struct_class()
     inner_struct['subkey2'] = 9
-    self.assertEqual([6, 'seven', True, False, None, inner_struct],
-                     list(struct['key5'].items()))
+    self.assertEqual(
+        [6, 'seven', True, False, None, inner_struct],
+        list(struct['key5'].items()),
+    )
     self.assertEqual(2, len(struct['key6'][0].values))
     self.assertEqual('nested_list', struct['key6'][0][0])
     self.assertEqual(True, struct['key6'][0][1])
@@ -942,10 +978,7 @@ class StructTest(unittest.TestCase):
 
     # According to documentation: "When parsing from the wire or when merging,
     # if there are duplicate map keys the last key seen is used".
-    duplicate = {
-        'key4': {'replace': 20},
-        'key5': [[False, 5]]
-    }
+    duplicate = {'key4': {'replace': 20}, 'key5': [[False, 5]]}
     struct.update(duplicate)
     self.assertEqual(1, len(struct['key4'].fields))
     self.assertEqual(20, struct['key4']['replace'])
@@ -962,16 +995,17 @@ class AnyTest(unittest.TestCase):
     msg_descriptor = msg.DESCRIPTOR
     all_types = unittest_pb2.TestAllTypes()
     all_descriptor = all_types.DESCRIPTOR
-    all_types.repeated_string.append(u'\u00fc\ua71f')
+    all_types.repeated_string.append('\u00fc\ua71f')
     # Packs to Any.
     msg.value.Pack(all_types)
-    self.assertEqual(msg.value.type_url,
-                     'type.googleapis.com/%s' % all_descriptor.full_name)
-    self.assertEqual(msg.value.value,
-                     all_types.SerializeToString())
+    self.assertEqual(
+        msg.value.type_url, 'type.googleapis.com/%s' % all_descriptor.full_name
+    )
+    self.assertEqual(msg.value.value, all_types.SerializeToString())
     # Tests Is() method.
     self.assertTrue(msg.value.Is(all_descriptor))
     self.assertFalse(msg.value.Is(msg_descriptor))
+
     # Unpacks Any.
     unpacked_message = unittest_pb2.TestAllTypes()
     self.assertTrue(msg.value.Unpack(unpacked_message))
@@ -984,8 +1018,9 @@ class AnyTest(unittest.TestCase):
     except AttributeError:
       pass
     else:
-      raise AttributeError('%s should not have Pack method.' %
-                           msg_descriptor.full_name)
+      raise AttributeError(
+          '%s should not have Pack method.' % msg_descriptor.full_name
+      )
 
   def testUnpackWithNoSlashInTypeUrl(self):
     msg = well_known_types_test_pb2.TestAny()
@@ -1012,16 +1047,17 @@ class AnyTest(unittest.TestCase):
     msg = any_pb2.Any()
     # Pack with a custom type URL prefix.
     msg.Pack(submessage, 'type.myservice.com')
-    self.assertEqual(msg.type_url,
-                     'type.myservice.com/%s' % submessage.DESCRIPTOR.full_name)
+    self.assertEqual(
+        msg.type_url, 'type.myservice.com/%s' % submessage.DESCRIPTOR.full_name
+    )
     # Pack with a custom type URL prefix ending with '/'.
     msg.Pack(submessage, 'type.myservice.com/')
-    self.assertEqual(msg.type_url,
-                     'type.myservice.com/%s' % submessage.DESCRIPTOR.full_name)
+    self.assertEqual(
+        msg.type_url, 'type.myservice.com/%s' % submessage.DESCRIPTOR.full_name
+    )
     # Pack with an empty type URL prefix.
     msg.Pack(submessage, '')
-    self.assertEqual(msg.type_url,
-                     '/%s' % submessage.DESCRIPTOR.full_name)
+    self.assertEqual(msg.type_url, '/%s' % submessage.DESCRIPTOR.full_name)
     # Test unpacking the type.
     unpacked_message = well_known_types_test_pb2.TestAny()
     self.assertTrue(msg.Unpack(unpacked_message))
@@ -1034,11 +1070,13 @@ class AnyTest(unittest.TestCase):
     msg = any_pb2.Any()
     msg.Pack(submessage, deterministic=True)
     serialized = msg.SerializeToString(deterministic=True)
-    golden = (b'\n4type.googleapis.com/google.protobuf.internal.TestAny\x12F'
-              b'\x1a\x05\n\x010\x10\x00\x1a\x05\n\x011\x10\x02\x1a\x05\n\x01'
-              b'2\x10\x04\x1a\x05\n\x013\x10\x06\x1a\x05\n\x014\x10\x08\x1a'
-              b'\x05\n\x015\x10\n\x1a\x05\n\x016\x10\x0c\x1a\x05\n\x017\x10'
-              b'\x0e\x1a\x05\n\x018\x10\x10\x1a\x05\n\x019\x10\x12')
+    golden = (
+        b'\n4type.googleapis.com/google.protobuf.internal.TestAny\x12F'
+        b'\x1a\x05\n\x010\x10\x00\x1a\x05\n\x011\x10\x02\x1a\x05\n\x01'
+        b'2\x10\x04\x1a\x05\n\x013\x10\x06\x1a\x05\n\x014\x10\x08\x1a'
+        b'\x05\n\x015\x10\n\x1a\x05\n\x016\x10\x0c\x1a\x05\n\x017\x10'
+        b'\x0e\x1a\x05\n\x018\x10\x10\x1a\x05\n\x019\x10\x12'
+    )
     self.assertEqual(golden, serialized)
 
   def testJsonStruct(self):
